/******************************************************************************

* File Name : DerefSigner.js

* Purpose : Retrieve public key, blind entries and get signed entries

* Creation Date : 09-03-2011

* Last Modified : Wednesday, July 20, 2011 PM01:21:57 HKT

*******************************************************************************/
//Constructor
derefSigner = function(serverURL,pubkey){
	this.e = str2bigInt("10001",16,0);
	this.N = str2bigInt(pubkey,16,0);
	this.logger = oDerefLogger;
	this.serverURL = serverURL;
}
derefSigner.prototype = {
	blind: function(url){
		var m = str2bigInt(hex_md5(url),16,0);
		var m_blind = multMod(m,powMod(this.r,this.e,this.N),this.N);
		return m_blind;
	},
	//Find a secure random generating function
	randomize: function(){
		this.r = str2bigInt(hex_md5(Math.random().toString()),16,0);
	},
	sign: function(msg){
		var m_blind_signed = this.sendMessage(this.serverURL+"sign.php",msg);
		return str2bigInt(m_blind_signed,16);
	},
	signAll: function(msg){
		var m_blind_signed = this.sendPostMessage(this.serverURL+"signAll.php",msg);
		return JSON.parse(m_blind_signed);
	},
	unblind:function(msg){
		return multMod(msg,inverseMod(this.r,this.N),this.N)
	},
	main: function(target,urls){
		this.randomize();
		var blindList = new Object();
		blindList["target"] = new Object();
		for(var i in target)
			blindList["target"][target[i]] = bigInt2str(this.blind(target[i]),16);

		blindList["init"] = new Array();
		for(var i = 0; i < urls.length;i++){
			blindList["init"][i] = new Object();
			for(var j in urls[i])
				blindList["init"][i][urls[i][j]]=(bigInt2str(this.blind(urls[i][j]),16));
		}
		var result = this.signAll(JSON.stringify(blindList));
		for(var i in result["target"])
			result["target"][i] = hex_md5(i+bigInt2str(this.unblind(str2bigInt(result["target"][i],16)),16));
		for(var i in result["init"]){
			for(var j in result["init"][i])
			result['init'][i][j] = hex_md5(j+bigInt2str(this.unblind(str2bigInt(result["init"][i][j],16)),16));
		}
		return result;
	},
	retrieveKey: function(url){
		var request = Components
					.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
					.createInstance();
		request.open("GET",url,false);
		request.send(null);
		if(request.status == 200){
			return request.responseText;
		}
	},
	sendPostMessage: function(url,m){
		try {
			m = "message="+m;
			var request = Components
						.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
						.createInstance();
			request.open("POST",url,false);
			request.setRequestHeader("Content-length",m.length);
			request.setRequestHeader("Content-type","application/x-www-form-urlencoded");
			request.send(m);
			if(request.status == 200){
				return request.responseText;
			}
		} catch( e ) {
			this.logger.Log(e);
			this.logger.Log("Bloom Filter not found");
		}
	},
	sendMessage: function(url,m){
		try {
			var request = Components
						.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
						.createInstance();
			request.open("GET",url+"?message="+m,false);
			request.send(null);
			if(request.status == 200){
				return request.responseText;
			}
		} catch( e ) {
			this.logger.Log(e);
			this.logger.Log("Bloom Filter not found");
		}
	},

	
}
oDerefSigner = new derefSigner();
