<?php
require_once("blind.php");
class BloomFilter {

	public $map; 
	public $len;
	public $salt;
	public $itemCount;
	public $hashCount;

	function __construct($len,$salt){
		$this->len = $len;
		$this->map = bitset_empty($this->len);
		$this->itemCount = 0;
		$this->hashCount = 32;
		$this->salt = $salt;
	}
	function ImportBase64($mapBase64){
		$map = base64_decode($mapBase64);
		$map = bitset_to_string($map);
		$this->len = strlen($map);
		for($i = 0; $i < $this->len; $i+=8){
			$this->map .= strrev(substr($map,$i,8)); 
		}
		$this->map = bitset_from_string($this->map);
	}
	function ExportBase64(){
		$bitmap = bitset_to_string($this->map);
		$result = "";
		for($i = 0; $i < $this->len; $i+=8){
			$result .= chr(bindec(substr($bitmap,$i,8))); 
		}
		$mapBase64 = base64_encode($result);
		return $mapBase64;
	}

	function hash($key){
		$result = array();
		$nextSalt = $this->salt;
		for($i = 0 ; $i < $this->hashCount; $i++){
			$result[] = (int)hexdec(substr(md5($nextSalt.$key),0,7))%$this->len;
			$nextSalt = md5($nextSalt);	
		}
		return $result;
	}

	function Add($key){
		$this->itemCount ++;
		foreach ($this->hash($key) as $pos){
			bitset_incl($this->map,$pos);
		}
	}

	function Has($key){
		foreach ($this->hash($key) as $pos) 
		if (!bitset_in($this->map,$pos)) 
			return false;
		return true;
	}

	function FalsePositiveRate(){
		$k = $this->hashCount;
		return pow((1-pow((1-1/$this->len),$k*$this->itemCount)),$k);
	}
	function ImportList($list){
		foreach($list as $line)
			$this->Add(sign(md5($line)));
	}

}
function ReadListing($fname){
	$result = array();
	$f = fopen($fname,"r") or die("Cannot Open ".$fname);
	while($line = fgets($f,4096)){
		$result[] = rtrim($line,"\r\n");
	}
	fclose($f);
	return $result;
}
function ReadPublic($fname){
	$result;
	$f = fopen($fname,"r") or die("Cannot Open ".$fname);
	while($line = fgets($f,4096)){
		$result = rtrim($line,"\r\n");
	}
	fclose($f);
	return $result;
}
function CreateBloomFilter($fname,$salt){
	$list = ReadListing($fname);
	$bf = new BloomFilter(55808,$salt);//bit
	$bf->ImportList($list);
	$mapBase64 = $bf->ExportBase64();
	echo $mapBase64."\n";
}
function CreateHashFilter($fname,$len){
	$filter = array();
	$list = ReadListing($fname);
	foreach($list as $url){
		$filter[substr(md5($url),0,$len)] = true;
		/*
		$hash = md5($url);
		$mask = 0;
		if($len == 1){
			if(intval($hash[0],16)%2 == 0)
				$filter[0] = true;
			else
				$filter[1] = true;
		}
		else if($len == 2){
			$mask = 3;
			$filter[intval(intval($hash[0],16) & $mask)] = true;
		}
		else if($len == 3){
			$mask = 7;
			$filter[intval(intval($hash[0],16) & $mask)] = true;
		}
		*/
	}
	return $filter;
}
function CreateSignedList($fname){
	$result = array();
	$list = ReadListing($fname);
	foreach($list as $url){
		$result[md5($url.sign(md5($url)))] = true;
	}
	return $result;
}
//Web Page
session_start();
$salt = rand(); 
$site = array();
$site[id] = session_id();
if($_GET[len]!= null)
	$site[len] = $_GET[len];
else
	$site[len] = 2;
$site[sensitive] = CreateHashFilter("sensitive",$site[len]);
$site[validScope] = CreateHashFilter("validScope",$site[len]);
$site[sensitiveList] = CreateSignedList("sensitive");
$site[validScopeList] = CreateSignedList("validScope");
$site[publicKey] = ReadPublic("n");
echo json_encode($site);
?>
